float4x4 MatrixW;
float4x4 MatrixWVP;

float4 MaterialAmbientColor;
float4 MaterialDiffuseColor;

float4 LightDiffuseColor;
float4 LightAmbientColor;
float3 LightDirection;

float3 EyeDirection;

texture MeshTexture;

sampler MeshTextureSampler = 
sampler_state{
	Texture = < MeshTexture >;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

// _VF[_

// gD[VF[_
struct VS_OUTPUT_Shader{
	float4 Pos			: POSITION;
	float4 Col			: COLOR0;
	float2 TexUV		: TEXCOORD0;
};

VS_OUTPUT_Shader VS_HDRShader(
	float4 Pos			: POSITION,
	float3 Normal		: NORMAL,
	float2 TexCoord0	: TEXCOORD0 )
{
	VS_OUTPUT_Shader Out = (VS_OUTPUT_Shader)0;
	Out.Pos = mul( Pos, MatrixWVP );

	float3 NormalW = mul( Normal, (float3x3)MatrixW );

	// XyL
	float3 Eye = normalize( EyeDirection - mul( Pos, MatrixW ) );
	float3 r = -Eye + 2.0f * dot( NormalW, Eye ) * NormalW;
	float4 Specular = pow( max( 0,dot( -LightDirection, r ) ), 10 ) * 5;

	// Œ蒸_̒_FvZ
	Out.Col = MaterialDiffuseColor * LightDiffuseColor * max( dot( -LightDirection, NormalW ), 0 )
			+ MaterialAmbientColor * LightAmbientColor
			+ Specular;

	// Kk
	Out.Col = Out.Col / ( Out.Col + 4 );
 
	Out.TexUV = TexCoord0;

	return Out;
}

// sNZVF[_
float4 PS_HDRShader( VS_OUTPUT_Shader In ) : COLOR0
{
	float4 s = In.Col * 4 / ( 1 - In.Col + 0.0001 ); // K(0.0001̓[p)
	float4 a = tex2D( MeshTextureSampler, In.TexUV ) * s;
	if( a.r + a.g + a.b > 5 ) a.b = 1; // nCCĝ1ɒt(5͓K)
	else a.b = min( a.b, 0.95 );
	
	return a;
}

float4 PS_HighlightShader( VS_OUTPUT_Shader In ) : COLOR0
{
	float4 a = tex2D( MeshTextureSampler, In.TexUV ) * In.Col;
	
	if( a.b > 0.99 ) a.rgb = 1; // nCCgo
	else a.rgb = 0;
	
	return a;
}

float4 PS_LowpassShader( VS_OUTPUT_Shader In ) : COLOR0
{
	float4 a = (
		  tex2D( MeshTextureSampler, In.TexUV + float2( -1.0f / 64, -1.0f / 64 ) )
		+ tex2D( MeshTextureSampler, In.TexUV + float2(  0.0f / 64, -1.0f / 64 ) ) * 2
		+ tex2D( MeshTextureSampler, In.TexUV + float2(  1.0f / 64, -1.0f / 64 ) )
		
		+ (
			  tex2D( MeshTextureSampler, In.TexUV + float2( -1.0f / 64,  0.0f / 64 ) )
			+ tex2D( MeshTextureSampler, In.TexUV + float2(  0.0f / 64,  0.0f / 64 ) ) * 2
			+ tex2D( MeshTextureSampler, In.TexUV + float2(  1.0f / 64,  0.0f / 64 ) )
		) * 2
		
		+ tex2D( MeshTextureSampler, In.TexUV + float2( -1.0f / 64,  1.0f / 64 ) )
		+ tex2D( MeshTextureSampler, In.TexUV + float2(  0.0f / 64,  1.0f / 64 ) ) * 2
		+ tex2D( MeshTextureSampler, In.TexUV + float2(  1.0f / 64,  1.0f / 64 ) )
	) / 16;
	
	return a;
}

// eNjbN
technique HDRShader{
	pass P0{
		VertexShader = compile vs_2_0 VS_HDRShader();
		PixelShader  = compile ps_2_0 PS_HDRShader();
	}
}

technique HighlightShader{
	pass P0{
		PixelShader  = compile ps_2_0 PS_HighlightShader();
	}
}

technique LowpassShader{
	pass P0{
		PixelShader  = compile ps_2_0 PS_LowpassShader();
	}
}
